@@ -19,7 +19,7 @@ from  flashcam.stream_enhancer import Stream_Enhancer
 
 
 from flashcam import v4lc
-from flashcam.v4lc import set_gem
+from flashcam.v4lc import set_gem, tune_histo
 
 from flashcam.mmapwr import mmread_n_clear
 
@@ -102,7 +102,7 @@ class Camera(BaseCamera):
         # -----------get parameters for DetMot, same for web as for all
         #print(config.CONFIG)
         #print( "AVERAGE I AM HAVING ",config.CONFIG['average'] )
-        framekind = config.CONFIG['framekind']
+        framekind    = config.CONFIG['framekind']
         average      = int(config.CONFIG['average'])
         threshold    = int(config.CONFIG['threshold'])
         blur         = int(config.CONFIG['blur'])
@@ -114,6 +114,8 @@ class Camera(BaseCamera):
         rotate180    = int(config.CONFIG['otate'])
         zoom         = int(config.CONFIG['zoom'])
 
+        print( "XY: ", config.CONFIG['x'] ,  config.CONFIG['y']  , speedx, speedy)
+
         # ------------------    to evaluate commands from web.py
         substract_background = False
         save_background = False
@@ -158,24 +160,28 @@ class Camera(BaseCamera):
         cc = v4lc.V4L2_CTL("/dev/video"+str(vidnum))
         capa = cc.get_capbilities()
 
-
-        # I CALL SET_GAM from
-        set_gem(cc, config.CONFIG['gain'],
-                config.CONFIG['expo'],
-                config.CONFIG['mmaga'])
+        if (config.CONFIG["histogram"]!=None) or \
+           (config.CONFIG["histogram"]==True):
+                print("i... HISTOGRAM ON ===> MANUAL EGM")
+                set_gem(cc, "def","def","def")
+        else:
+            # I CALL SET_GAM from
+            set_gem(cc, config.CONFIG['gain'],
+                    config.CONFIG['expo'],
+                    config.CONFIG['mmaga'])
 
 
         # very stupid camera    ZC0303 Webcam
         if "exposure" in capa:
             exposure = cc.get_exposure()
             exposuredef = cc.getdef_exposure()
-            print(f"i... EXPOAUTO == {exposure} vs def={exposuredef}; ")
+            print(f"i... EXPOAUTO (top) == {exposure} vs def={exposuredef}; ")
 
 
         if "exposure_auto" in capa:
             expo_auto = cc.get_exposure_auto()
             expo_autodef = cc.getdef_exposure_auto()
-            print(f"i... EXPOAUTO == {expo_auto} vs def={expo_autodef}; ")
+            print(f"i... EXPOAUTO (TOP) == {expo_auto} vs def={expo_autodef}; ")
 
         if "exposure_absolute" in capa:
             exposure_absolute = cc.get_exposure_absolute()
@@ -266,7 +272,7 @@ class Camera(BaseCamera):
 
 
                     #------------------------------ BRUTAL ------------test exposure V4L
-                    if (BaseCamera.nframes % 10==0):
+                    if (BaseCamera.nframes % 100==0):
                         cc = v4lc.V4L2_CTL("/dev/video"+str(vidnum))
                         capa = cc.get_capbilities()
                         #cc.refresh()
@@ -282,7 +288,7 @@ class Camera(BaseCamera):
                         if "exposure_auto" in capa: # notauto? alway show _absolute
                             expo_auto = cc.get_exposure_auto()
                             expo_autodef = cc.getdef_exposure_auto()
-                            print(f"i... EXPOAUTO == {expo_auto} vs def={expo_autodef}; ")
+                            print(f"i...                   EXPOAUTO (inloop) == {expo_auto}/{expo_autodef}  [{BaseCamera.nframes%3}]; ",end="\r")
 
                         if "exposure_absolute" in capa:
                             exposure_absolute = cc.get_exposure_absolute()
@@ -300,18 +306,26 @@ class Camera(BaseCamera):
                     #=========== like ZOOM
                     #=========== THEN CALCULATE HISTO =====
                     #=========== THEN do other stuff
+
+
+                    # 1. rotate (+translate of the center)
+                    # 2. zoom (+translate the center)
+                    # 3. histogram !!!here
+                    # 4. speed
+                    #  others
+
                     # senh has a frame now
                     if rotate180!=0:   # rotate earlier than zoom
                         senh.rotate180( rotate180 ) #arbitrary int angle
 
                     if zoom!=1:
-                    #if int(config.CONFIG['zoom'])!=1:
                         try:
                             crocfg = os.path.expanduser("~/.config/flashcam/cross.txt")
                             cross_dx, cross_dy  = None, None
                             if os.path.exists(crocfg):
                                 with open(crocfg) as f:
                                     cross_dx, cross_dy  = [int(x) for x in next(f).split()]
+                                    #senh.zoom( zoom ,0,0 )
                                     senh.zoom( zoom ,cross_dx, cross_dy )
                         except Exception as e:
                             print("!... Problem ar cross.txt file:",e)
@@ -319,9 +333,23 @@ class Camera(BaseCamera):
                     # ----------  I need to calculate histogram before labels...
                     if histogram: # just calculate a number on plain frame
                         hmean = senh.histo_mean( )
+                        #print("i... histo value:", hmean)
+                        tune_histo(cc, hmean )
 
                     # ---------- before anything - we decode the web command EXECUTE EXECUTION
 
+                    # - compensate for speed of the sky
+                    if ((speedx!=0) or (speedy!=0)) \
+                    and ((abs(speedx)>1) or (abs(speedy)>1)):
+                        senh.translate( speedx, speedy)
+
+                    if restart_translate:
+                        senh.reset_camera_start()
+                        restart_translate = False
+
+
+                    # ------------------------- commands comming from web.py----
+
                     expression,value = mmread_n_clear( )
                     #print(f"i... expression == {expression}  value == {value}")
                     # --- fixed_image
@@ -330,10 +358,6 @@ class Camera(BaseCamera):
                     except:
                         print("X... EXEC FAIL",expression,value)
 
-                    # print("i... fixed:", fixed_image)
-                    # print(".... substract_background", substract_background, not(substract_background) )
-                    # globals()["substract_background"] = value
-
 
 
                     if save_background:
@@ -350,7 +374,8 @@ class Camera(BaseCamera):
                         senh.subtract()
 
                     # - compensate for speed of the sky
-                    if (speedx!=0) or (speedy!=0):
+                    if ((speedx!=0) or (speedy!=0)) and ((abs(speedx)<1) and (abs(speedy)<1)):
+                        print(f"speed translate {speedx} {speedy}")
                         senh.translate( speedx, speedy)
 
                     if restart_translate:
