@@ -355,6 +355,7 @@ class Stream_Enhancer:
 
 
     def rotate180(self, angle):
+        #print(f" rotate {angle}")
         if angle == 0:
             return
         elif angle == 180:
@@ -388,7 +389,13 @@ class Stream_Enhancer:
         self.camera_start = dt.datetime.now()
 
     def translate(self, speedx = -0.35,  speedy = -0.015 ):
-        self.camera_age = (dt.datetime.now() - self.camera_start ).total_seconds()
+        #print(f"translate {speedx} {speedy}")
+        if ( abs(speedx)>1 ) or ( abs(speedy)>1 ):
+            self.camera_age = 1
+        else:
+            self.camera_age = (dt.datetime.now() - self.camera_start ).total_seconds()
+
+        # --- for larger speeds
         dwidth = speedx * self.camera_age  #negative up
         dheight = speedy * self.camera_age  # positive down
         T = np.float32([[1, 0, dwidth], [0, 1, dheight]])
@@ -603,8 +610,13 @@ class Stream_Enhancer:
         #print(f"i... mimax mean  {hist_gray.min():4.1f},  {hist_gray.max():4.1f}, {hist_gray.mean():4.1f} ... {mean}" )
         #print( hist_gray[:9],  bins[:9]  , len(hist_gray), len(bins) , type(hist_gray), type(bins) )
 
-
-        mean = np.dot( bins, hist_gray )[0][0]/hist_gray.sum()/2.55
+        # strange problem WITH 046d:080f  Webcam C120
+        #print(f"                     bins={len(bins)}  len={len(hist_gray)}")
+        sumhist = hist_gray.sum()
+        #print(f"                     sumhist = {sumhist}")
+        if sumhist<1:
+            return 100 # I DONT KNOW WHAT IS THIS
+        mean = np.dot( bins, hist_gray )[0][0]/sumhist/2.55
         #print("MEAN",mean)
         #print( "LEN TYPE", len(mean),type(mean) )
 
@@ -632,7 +644,7 @@ class Stream_Enhancer:
         #    centerX+= x
         #if not y is None:
         #    centerY+= y
-        if (x!=0) or (y!=0):
+        if (x!=None) or (y!=None):
             dwidth = -x    #negative up
             dheight = -y   # positive down
             T = np.float32([[1, 0, dwidth], [0, 1, dheight]])
