@@ -1,5 +1,7 @@
 #!/usr/bin/env python3
 
+# there is tuning of the size------
+
 # import the necessary packages
 
 #  ls zen_192.168.0.91_20211105_* | while read line; do echo file \'$line\'; done | ffmpeg -protocol_whitelist file,pipe -f concat -i - -c copy zen_192.168.0.91_20211105_.avi
@@ -36,6 +38,13 @@ import sys
 from flashcam.stream_enhancer import Stream_Enhancer
 import webbrowser
 
+import math
+
+
+global centerX1, centerY1
+global FILE_USERPASS
+FILE_USERPASS = "~/.flashcam_upw"
+
 @contextmanager
 def timeout(time):
     # register a function to raise a TimeoutError on the signal
@@ -89,7 +98,7 @@ def get_stream3( videodev, passfile ):
             u= w1[0].strip()
             p= w1[1].strip()
     except:
-        print("NO PASSWORD FILE  ", passfile)
+        print("NO PASSWORD FILE (gs3)  ", passfile)
         sys.exit(0)
 
     print("D... capturing from /{}/".format(videodev) )
@@ -201,17 +210,36 @@ def display3(videodev, save = False,
             key = cv2.waitKey(1)
 
 
+# ================================================================================================
 
 # ================================================================================================
 
 def display2(videodev, save = False,
              passfile="~/.pycamfw_userpass",
-             rotate = 0):
+             rotate = 0, vof = 99):
     """
     """
     #sname,sfilenamea,sme,sfilenamea,sfilenamea,sfourcc,saviout
 
     sme = socket.gethostname()
+    #frame = None
+    global centerX1, centerY1, clicked
+    global FILE_USERPASS
+    centerX1, centerY1, clicked = 0,0, False
+
+    def MOUSE(event, x, y, flags, param):
+        global centerX1,centerY1, clicked
+        if event == cv2.EVENT_LBUTTONDOWN:
+            clicked = not clicked
+        if event == cv2.EVENT_MOUSEMOVE:
+            if not(clicked):
+                centerX1,centerY1 = x,y
+            #print('({}, {})'.format(x, y))
+            #imgCopy = frame.copy()
+            #cv2.circle(imgCopy, (x, y), 5, (255, 0, 0), -1)
+            #cv2.imshow('image', imgCopy)
+
+
     def setupsave():
         sname = "rec"
         sname = videodev
@@ -235,8 +263,23 @@ def display2(videodev, save = False,
 
     def get_stream():
         # localuser
+        global FILE_USERPASS
         stream = None # i return
         u,p=getpass.getuser(),"a"
+
+        if "passfile" in locals():
+            if passfile is None:
+                print(f"i... nO passfile...trying {videodev} , {passfile}")
+                passfile = videodev.strip("http://")
+                print("i... TRYING", passfile)
+        else:
+            passfile = videodev.strip("http://")
+            passfile = passfile.strip("/video")
+            passfile = passfile.split(":")[0]
+            passfile = f"{FILE_USERPASS}_{passfile}"
+            print(f"i... TRYING {videodev} PASS: {passfile}")
+
+
         try:
             with open( os.path.expanduser(passfile) ) as f:
                 print("YES---> PASSWORD FILE  ", passfile )
@@ -244,7 +287,7 @@ def display2(videodev, save = False,
                 u= w1[0].strip()
                 p= w1[1].strip()
         except:
-            print("NO PASSWORD FILE  ", passfile)
+            print("NO PASSWORD FILE (gs) ")
 
 
         print("D... capturing from web:/{}/".format(videodev) )
@@ -261,14 +304,16 @@ def display2(videodev, save = False,
         print("D... stream ok2 - request.urlopen (disp)")
         ok = False
         try:
-            stream = urllib.request.urlopen(request, timeout=5)
+            stream = urllib.request.urlopen(request, timeout=3) # timeout to 7 from 5 sec.
             ok = True
             print("D... stream ok3")
         except urllib.error.HTTPError as e:
             print("Server Offline1? ",e)
+            print(videodev)
             #do stuff here
         except urllib.error.URLError as e:
             print("Server Offline2? ",e)
+            print(videodev)
             #do stuff here
         except:
             ok = False
@@ -283,7 +328,12 @@ def display2(videodev, save = False,
     io_none = 0 # to reset stream
     sfilename = ""  # move up to limi # of AVI files.... tst?
     sfilenamea = ""
-    stream_length = 1024*50
+
+    stream_length = 1024*50  # i had 50k all the time from 1st working version
+    stream_length = 1024*15  #
+
+
+
     if save:
         sfilenamea,saviout = setupsave()
     while True: #==================== MAIN LOOP =================
@@ -358,7 +408,7 @@ def display2(videodev, save = False,
                         frame = cv2.imdecode(np.frombuffer(jpg, dtype=np.uint8), cv2.IMREAD_COLOR)
                         ret_val= 1
                         io_none = 0
-                        stream_length = b+2-a
+                        stream_length = int( (b+2-a)/2 ) # expected length
                     else:
                         ret_val = 0
                         # print("D...                ok frame http",oi,len(bytex) )
@@ -404,10 +454,25 @@ def display2(videodev, save = False,
         FPS = 0
         FPSlast = 0
         frames_total = 0
+        frame_num = 0
 
         senh = Stream_Enhancer()
         saved_jpg = False
 
+        zoomme=1
+        expande=1
+        rorate180 = False
+
+
+        # measurements (distance)
+        measure = 0 #1.7
+        measure_fov = vof # config.CONFIG['vof'] #
+
+
+        # -see the values sent from the webpy
+        webframen = "-------" # frame number from web.py()
+        webframetime = ""
+
         while connection_ok: #========================================================
             # read the frame from the camera and send it to the server
             #time.sleep(0.05)
@@ -425,21 +490,42 @@ def display2(videodev, save = False,
                                 bytex += stream.read(stream_length)
                                 a = bytex.find(b'\xff\xd8') #frame starting
                                 b = bytex.find(b'\xff\xd9') #frame ending
+                                ttag = bytex.find(f"#FRAME_ACQUISITION_TIME".encode("utf8")) #frame ending
+                                webframen = " "*7
+                                webframetime = " "*23
+                                if ttag!=-1:
+                                    #print(f"i... FRACQT: /{ttag}/ /{bytex[ttag:ttag+32]}/----------------")
+                                    webframen = bytex[ttag:ttag+32+23].decode("utf8")
+                                    webframen = webframen.split("#")
+                                    #print(len(webframen), webframen)
+                                    webframen,webframetime = webframen[2],webframen[3]
+
+                                    #print( webframen )
+                                    #print( webframetime)
+
                                 if a != -1 and b != -1:
                                     jpg = bytex[a:b+2]
                                     BPS+=len(jpg)/1024
                                     if len(jpg)>0:
                                         FPS+=1
                                     bytex = bytex[b+2:]
+                                    # just a test.... if I can append
+                                    #jpg = jpg+b'#FRAME_ACQUISITION_TIME#'+f"a".encode("utf8")
                                     frame = cv2.imdecode(np.frombuffer(jpg, dtype=np.uint8), cv2.IMREAD_COLOR)
-                                    stream_length = b+2-a
+                                    frame_num+=1
+                                    #stream_length = b+2-a
+                                    stream_length = int( (b+2-a)*0.9 ) # expected length
 
                                     ret_val=1
-                                    print("   {:4.1f}Mb/s {}fps {} {}".format(
+                                    print("{:.1f}k #{:06d}/{} {:4.1f}Mb/s {}fps rec{} web{} {}".format(
                                         # len(bytex)/1024,
+                                        stream_length/1024,
+                                        frame_num,
+                                        webframen,
                                         BPSlast*8/1024,
                                         FPSlast,
-                                        str(datetime.datetime.now())[:-5],
+                                        str(datetime.datetime.now())[11:-5],
+                                        webframetime[11:],
                                         sfilenamea.replace('/home/','')
                                     ) , end="\r")
 
@@ -472,6 +558,7 @@ def display2(videodev, save = False,
                 if (ret_val == 0) or (type(frame)=="NoneType"):
                     print("Not a good frame", type(frame), end="\r")
                     continue
+                frame = frame
                 (w, h, c) = frame.shape
                 frame_area = w*h
                 motion_det = False
@@ -492,11 +579,33 @@ def display2(videodev, save = False,
                     # https://stackoverflow.com/a/52776376
                     cv2.setWindowProperty(wname, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
                     bit_fs = 0
-                    if sme in ['gigavg','vaio']:
+                    if sme in ['gigavg','vaio']: # strange behavior on some PC concerning resize...(checked in MS with vadim)
                         bit_fs =1
                     cv2.setWindowProperty(wname, cv2.WND_PROP_FULLSCREEN,  bit_fs)
                     cv2.resizeWindow(wname, frame.shape[1], frame.shape[0] )
 
+
+                if zoomme>1: # FUNNY - it continues to zoom where the mouse pointer is !!!!
+                    if senh.add_frame(frame):
+                        # print("avi..")
+                        senh.setbox(f"z {zoomme}",  senh.scale)
+                        w,h,c = frame.shape
+                        # print(w-centerY1, h-centerX1)
+                        senh.zoom( zoomme , int(centerX1-h/2), int(centerY1-w/2)  )
+                        frame = senh.get_frame(  )
+
+                # if rotate180:
+                #     if senh.add_frame(frame):
+                #         senh.setbox(f"ROT",  senh.rot)
+                #         # w,h,c = frame.shape
+                #         # print(w-centerY1, h-centerX1)
+                #        senh.rotate180( 180 )
+                #        frame = senh.get_frame(  )
+                if rotate == 180:
+                    frame = cv2.rotate(frame, cv2.ROTATE_180)
+                    #frame = cv2.rotate(frame, cv2.ROTATE_90_CLOCKWISE)
+
+
                 if save:
                     # print("AVI")
                     saviout.write(frame)
@@ -510,50 +619,217 @@ def display2(videodev, save = False,
                         senh.setbox(f"JPG",  senh.jpg)
                         frame = senh.get_frame(  )
 
-                if rotate == 180:
-                    frame = cv2.rotate(frame, cv2.ROTATE_180)
-                    #frame = cv2.rotate(frame, cv2.ROTATE_90_CLOCKWISE)
+                # MEASUREMENT
+                if measure>0:
+                    h,w = frame.shape[0], frame.shape[1]
+                    # measure_fov = 110.5 # notebook
+
+                    # approximative, not precise... 5%
+                    radians_per_pixel =  (measure_fov /180*math.pi) /w
+
+                    # rad per pix * unit distance * better
+                    radians_per_pixel2 =  math.tan(measure_fov /180*math.pi/2/w)
+
+
+                    radians_per_pixel2/=zoomme
+
+                    #print(f"RPPX {radians_per_pixel}  {radians_per_pixel2} ")
+
+                    # now arbitrarily define 1 meter..like.. 100px =>
+                    # alpha = 100*radians_per_pixel
+                    # b = 1m / math.tan( alpha )
+
+                    def get_order(dist = 1.7): #  determine order that fits
+                        wide=0.01
+                        while True:
+                            wide*=10
+                            pixwid = math.atan( wide/dist ) / radians_per_pixel2
+                            alpha = pixwid * radians_per_pixel2
+                            if pixwid>w/2*0.8: # not full rANGE
+                                wide/=10
+                                pixwid = math.atan( wide/dist ) / radians_per_pixel2
+                                alpha = pixwid * radians_per_pixel2
+                                break
+                        order = wide
+                        row = []
+
+                        while True:
+                            wide+=order
+                            pixwid = math.atan( wide/dist ) / radians_per_pixel2
+                            alpha = pixwid * radians_per_pixel2
+                            if pixwid>w/2*0.8: # not full rANGE:
+                                wide-=order
+                                pixwid = math.atan( wide/dist ) / radians_per_pixel2
+                                alpha = pixwid * radians_per_pixel2
+                                break
+                            else:
+                                row.append(wide)
+                        #-----
+                        #-----
+                        row=row[::-1] # revert - we want Big to small
+                        row.append(order)
+                        if len(row)<4:
+                            in0 = row[-1]/2
+                            in1 = row[-1]/5
+                            #in2 = row[-1]/10
+                            row.append( in0 )
+                            row.append( in1 )
+                            #row.append( in2 )
+                        return row
+
+
+                    def one_mark( dist = 1.7, wide=[1,2] ):
+                        # global measure_fov
+                        #h,w = frame.shape[0], frame.shape[1]
+                        # pixel distance of halfwidth
+
+                        # alpha = pixwid * radians_per_pixel2
+                        # dist = wide/math.tan( alpha)
+                        # I need to calculate 1m
+                        level=0
+                        #print("XXXXX",wide)
+                        for iwide in wide:
+                            pixwid = math.atan( iwide/dist ) / radians_per_pixel2
+                            alpha = pixwid * radians_per_pixel2
+
+                            #print(f" {radians_per_pixel}radpp {pixwid}   {wide}m <- {dist} ")
+                            step = 0
+                            mX,mY = int(w/2),int(h/2)
+                            mY= mY+level*step - 60
+                            yA, yB = mY, mY
+
+                            xA = mX
+                            xB = mX + int(pixwid)
+                            color = (0,255,0) # BGR
+                            if level==0:
+                                cv2.line(frame,
+                                         (int(xA), int(yA)), (int(xB), int(yB)),
+                                         color, 1)
+                            cv2.line(frame,
+                                     (int(xA), int(yA+2)), (int(xA), int(yA-2)),
+                                     color, 1)
+                            cv2.line(frame,
+                                     (int(xB), int(yB+2)), (int(xB), int(yB-2)),
+                                     color, 1)
+
+
+                            unit = "m"
+                            if level>0: unit=""
+                            unit2 = "m"
+
+                            if iwide<1:
+                                if zoomme==1:
+                                    iwide = round(iwide*10)/10
+                                else:
+                                    iwide = round(iwide*100)/100
+                            else:
+                                iwide = round(iwide)
+
+
+                            if str(iwide)[:2] == "0.":iwide=str(iwide)[1:]
+
+                            cv2.putText(frame, f"{iwide}{unit}",
+                                        (int(xB-10), int(  mY -7 )),  # little rightx a bit up y
+                                        cv2.FONT_HERSHEY_SIMPLEX, 0.35, color, 1)
+                            if level >= 0:
+                                cv2.putText(frame, f"  at {dist} {unit2}",
+                                            (int(xA-90), int(  mY+5 )),  # little rightx a bit up y
+                                            cv2.FONT_HERSHEY_SIMPLEX, 0.35, color, 1)
+                                cv2.putText(frame, f"  FOV {measure_fov:.1f}deg",
+                                            (int(xA-90), int(  mY+15 )),  # little rightx a bit up y
+                                            cv2.FONT_HERSHEY_SIMPLEX, 0.35, color, 1)
+                            level+=1
+
+                    order = get_order(dist=measure)
+                    one_mark( dist=measure ,    wide=order )
+
+
+
 
                 cv2.imshow(rpi_name, frame ) # 1 window for each RPi
+                if expande >1 :
+                    cv2.resizeWindow( rpi_name, int(expande*frame.shape[1]), int(expande*frame.shape[0]) )
+                    #print(frame.shape)
                 #cv2.setWindowProperty(rpi_name, cv2.WND_PROP_TOPMOST, 1)
                 # cv2.namedWindow(rpi_name, cv2.WINDOW_GUI_EXPANDED)
                 # time.sleep(0.2)
+                cv2.setMouseCallback(rpi_name, MOUSE)
                 key = cv2.waitKey(1)
 
+
+                # print(f"{centerX1} {centerY1}")
+
                 if (not frame is None) and (rpi_name!="") and (key == ord('h')):
                     print("h PRESSED! - ")
                     print("""
  h ... show this help
- a ... SAVING AVI
- z ... stopping AVI
- w ... openning web browser
+
+ a ... SAVE AVI
+ A ... stopping AVI
+ z ... cycle zoom 2x (mouse click fixes the center)
+ Z ... zoom 1x
+ x ... expand 2x ... buggy
+
  s ... saving JPG
+ r ... rotate by 180 degrees
+
+ w ... openning web browser
  q ... quit
 """)
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('w')):
-                    print("W PRESSED! - openning web browser")
+                    print("w PRESSED! - openning web browser")
                     webbrowser.open( videodev.replace("/video","" ) ) # BRUTAL
 
+                if (not frame is None) and (rpi_name!="") and (key == ord('r')):
+                    print("r PRESSED! - rotate change")
+                    if rotate == 180:
+                        rotate = 0
+                    else:
+                        rotate = 180
+
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('a')):
-                    print("A PRESSED! - saving AVI")
+                    print("a PRESSED! - saving AVI")
                     save = True
                     sfilenamea,saviout = setupsave()
                     print(">>>", sfilenamea )
 
-                if (not frame is None) and (rpi_name!="") and (key == ord('z')):
-                    print("Z PRESSED! - STOPPING stopping saving AVI")
+                if (not frame is None) and (rpi_name!="") and (key == ord('A')):
+                    print("A PRESSED! - STOPPING stopping saving AVI")
                     save = False
                     sfilenamea = ""
 
+                if (not frame is None) and (rpi_name!="") and (key == ord('z')):
+                    print("z PRESSED! - ZOOM 2")
+                    zoomme*= 2
+                    if zoomme>16:
+                        zoomme=1
+                    sfilenamea = ""
+
+                if (not frame is None) and (rpi_name!="") and (key == ord('Z')):
+                    print("Z PRESSED! - ZOOM ended")
+                    zoomme = 1
+                    sfilenamea = ""
+
                 if (not frame is None) and (rpi_name!="") and (key == ord('q')):
-                    print("Q PRESSED!")
+                    print("q PRESSED!")
                     sys.exit(1)
 
+
+
+                if (not frame is None) and (rpi_name!="") and (key == ord('x')):
+                    print("x PRESSED! - expand 2")
+
+                    if expande == 2:
+                        expande = 1
+                    else:
+                        expande = 2
+
+
                 saved_jpg = False
                 if (not frame is None) and (rpi_name!="") and (key == ord('s')):
-                    print("S PRESSED!")
+                    print("s PRESSED!")
                     sname = "snapshot"
                     saved_jpg = True
                     sfilename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
@@ -567,6 +843,23 @@ def display2(videodev, save = False,
                         print('Image is successfully saved as file.', sfilename)
 
 
+                if (not frame is None) and (rpi_name!="") and (key == ord('n')):
+                    print("m PRESSED! - measure far")
+                    measure = round(10*measure/1.15)/10
+                    if measure <1:
+                        measure = 1
+
+                if (not frame is None) and (rpi_name!="") and (key == ord('m')):
+                    print("n PRESSED! - measure close")
+                    if measure == 0:
+                        measure = 1
+                    measure = round(10*measure*1.15)/10
+                    if measure >10000:
+                        measure = 10000
+
+                if (not frame is None) and (rpi_name!="") and (key == ord('M')):
+                    print("M PRESSED! - DEmeasure")
+                    measure = 0
 
 
 
