@@ -1,3 +1,4 @@
+
 #!/usr/bin/env python3
 '''
  on https://github.com/TheStaticTurtle/RemoteV4L2-CTL/blob/master/remote_v4l2ctl/utils.py
@@ -21,6 +22,12 @@ import itertools # duplicite list removeal
 
 import sys
 
+
+import time # tuning has a delay from camera chip.... oscilates...
+
+import random # i need to skip some tune frames
+
+
 class V4L2_Control:
     """docstring for V4L2_CTL"""
 
@@ -267,24 +274,92 @@ class V4L2_CTL():
 
 
 
+def get_gem( cc, capa ):
+    # capa = cc.get_capbilities()
+
+    agm = None
+    aex = None
+    aea = None
+    aga = None
+
+    if "gamma" in capa:
+        gm = cc.get_gamma()
+        gmd = cc.getdef_gamma()
+        minm = cc.getmin_gamma()
+        maxm = cc.getmax_gamma()
+        gm10 = (gm-minm)/(maxm-minm)
+        agm = (gm,gmd,minm,maxm,gm10)
+        print(f"I... gamma             {gm:4d};  range({minm:4d},{maxm:5d}) def: {gmd} {gm10:.5f}")
+
+    if "exposure_absolute" in capa:
+        ex = cc.get_exposure_absolute()
+        exd=  cc.getdef_exposure_absolute()
+        mine = cc.getmin_exposure_absolute()
+        maxe = cc.getmax_exposure_absolute()
+        ex10 = (ex-mine)/(maxe-mine)
+        aex = (ex,exd,mine,maxe,ex10)
+        print(f"I... exposure_absolute {ex:4d};  range({mine:4d},{maxe:5d}) def: {exd} {ex10:.5f}")
+
+    if "exposure_auto" in capa:
+        ea = cc.get_exposure_auto()
+        ead=  cc.getdef_exposure_auto()
+        minea = cc.getmin_exposure_auto()
+        maxea = cc.getmax_exposure_auto()
+        aea = (ea,ead,minea,maxea)
+        print(f"I... exposure auto     {ea:4d};  range({minea:4d},{maxea:5d}) def: {ead}")
+
+    # A STRANGE CAMERA
+    if "exposure" in capa:
+        ex = cc.get_exposure()
+        exd=  cc.getdef_exposure_absolute()
+        mine = cc.getmin_exposure()
+        maxe = cc.getmax_exposure()
+        gm10 = (ex-mine)/(maxe-mine)
+        aex = (ex,exd,mine,maxe,ex10)
+        print(f"I...  exposure         {ex:4d};  range({mine:4d},{maxe:5d}) def: {exd} {ex10:.5f}")
+
+    if "gain" in capa:
+        ga = cc.get_gain()
+        gad = cc.getdef_gain()
+        ming = cc.getmin_gain()
+        maxg = cc.getmax_gain()
+        ga10 = (ga-ming)/(maxg-ming)
+        aga = (ga,gad,ming,maxg,ga10)
+
+        print(f"I...  gain             {ga:4d};  range({ming:4d},{maxg:5d}) def: {gad} {ga10:.5f}")
 
 
-def set_gem(cc, gain, expo, mmaga):
+    return aea,aex,aga,agm
+
+
+
+
+
+
+
+
+def set_gem(cc, gain=None, expo=None, mmaga=None):
+
     capa = cc.get_capbilities()
+    aea,aex,aga,agm = get_gem(cc, capa)
+    if aex!=None: ex,exd,mine,maxe,ex10 = aex
+    if agm!=None: gm,gmd,minm,maxm,gm10 = agm
+    if aga!=None: ga,gad,ming,maxg,ga10 = aga
+
+
+
 
     if not expo is None:
-        if (expo == "auto") or (expo == "def"):
+        if (expo == "auto"):
             if "exposure_absolute" in capa:
                 print("D... AUTO EXPOSURE ON")
-                cc.setdef_exposure_auto() # doesnt do default 3
-                print("ex def",cc.get_exposure_auto())
                 cc.setdef_exposure_auto()  # i thik he knows what is auto
         elif expo !=-1:
             if "exposure_absolute" in capa:
                 if "exposure_auto" in capa:
                     print("D... AUTO EXPOSURE OFF")
                     cc.setdef_exposure_auto() # doesnt do default 3
-                    print("ex def",cc.get_exposure_auto())
+                    #print("ex def",cc.get_exposure_auto())
                     cc.set_exposure_auto(1)  # I just a guess, 1 may be manual, 3
 
 
@@ -293,10 +368,16 @@ def set_gem(cc, gain, expo, mmaga):
                     maxe = cc.getmax_exposure_absolute()
                     print(f"i... current exposure {ex}   range {mine}, {maxe}")
 
-                    ex = int( expo * (maxe-mine)+mine)
-                    print("i... new = ",ex)
-                    if ex>maxe: ex=maxe
-                    if ex<mine: ex=mine
+                    if (expo == "def"):
+                        cc.setdef_exposure_absolute()
+                        print("i... new ex= default")
+                    else:
+                        ex = int( expo * (maxe-mine)+mine)
+                        if expo>1: ex+=1
+                        if expo<1: ex-=1
+                        if ex>maxe: ex=maxe
+                        if ex<mine: ex=mine
+                        print(f"i... new ex1= {ex};   {expo}")
 
                     cc.set_exposure_absolute(ex)
             # very stupid wabcam
@@ -307,9 +388,11 @@ def set_gem(cc, gain, expo, mmaga):
                 print(f"i... current exposure {ex}   range {mine}, {maxe}")
 
                 ex = int( expo * (maxe-mine)+mine)
-                print("i... new = ",ex)
+                if expo>1: ex+=1
+                if expo<1: ex-=1
                 if ex>maxe: ex=maxe
                 if ex<mine: ex=mine
+                print("i... new ex2= ",ex)
 
                 cc.set_exposure(ex)
             else:
@@ -326,10 +409,11 @@ def set_gem(cc, gain, expo, mmaga):
                 print(f"i... current gain {ga}   range {ming}, {maxg}")
 
                 ga = int( gain * (maxg-ming)+ming)
-                print("i... new = ",ga)
+                if gain>1: ga+=1
+                if gain<1: ga-=1
                 if ga>maxg: ga=maxg
                 if ga<ming: ga=ming
-
+                print("i... new ga = ",ga)
                 cc.set_gain(ga)
             else:
                 print("X... gain noit in capacities")
@@ -340,17 +424,17 @@ def set_gem(cc, gain, expo, mmaga):
                 cc.setdef_gamma()
         elif mmaga !=-1:
             if "gamma" in capa:
-                ga = cc.get_gamma()
-                ming = cc.getmin_gamma()
-                maxg = cc.getmax_gamma()
-                print(f"i... current gamma {ga}   range {ming}, {maxg}")
+                gm = cc.get_gamma()
+                minm = cc.getmin_gamma()
+                maxm = cc.getmax_gamma()
+                print(f"i... current gamma {gm}   range {minm}, {maxm}")
 
-                ga = int( mmaga * (maxg-ming)+ming)
-                print("i... new = ",ga)
-                if ga>maxg: ga=maxg
-                if ga<ming: ga=ming
+                gm = int( mmaga * (maxm-minm)+minm)
+                if ga>maxg: gm=maxm
+                if ga<ming: gm=minm
+                print("i... new gm= ",gm)
 
-                cc.set_gamma(ga)
+                cc.set_gamma(gm)
             else:
                 print("X... gamma not in capacities")
 
@@ -387,6 +471,13 @@ def test_func():
     print("i... TESTING function func")
     assert func() == True
 
+
+
+
+
+
+
+
 #------------------------------------------------------------------------------------------------
 
 def get_resolutions(vidnum):
@@ -405,18 +496,83 @@ def get_resolutions(vidnum):
 
 
 
-def tune_histo(cc, h_avg):
 
-    glow,ghigh = 100, 120
+
+
+#==================================== WORKS
+
+def tune_histo(cc, h_avg, limitgamma=150):
+
+    glow,ghigh = 16,28
+
+    dg= (ghigh-glow)/2
+    med = (ghigh+glow)/2
+
+    if (h_avg>=glow) and (h_avg<=ghigh):
+        return
+    h_avg= int(h_avg)
+
+    dist = (h_avg- med)/dg # distance in #of sigma
+    if abs(dist)<=1:
+        return
 
     capa = cc.get_capbilities()
+    aea,aex,aga,agm = get_gem(cc, capa)
+    if aex!=None: ex,exd,mine,maxe,ex10 = aex
+    if agm!=None: gm,gmd,minm,maxm,gm10 = agm
+    if aga!=None: ga,gad,ming,maxg,ga10 = aga
+
+
+    ok = False
+    if dist<-1:
+        if (ex<0.2) and(random.randint(0,10)<7): # I skip   frames to stop oscilation
+            return
+        if aex!=None:
+            if ex10<1:
+                set_gem(cc, expo=ex10*1.01+0.001)
+                ok = True
+        if not ok:
+            if aga!=None:
+                if ga10<1:
+                    set_gem(cc, gain=ga10*1.01+0.001)
+                    ok = True
+
+    if dist>1:
+        if (ex10<0.2) and (random.randint(0,10)<2): # I skip   frames to stop oscilation
+            return
+        if aex!=None:
+            if ex10>0:
+                set_gem(cc, expo=ex10*0.99-0.001)
+                ok = True
+        if not ok:
+            if aga!=None:
+                if ga10>0:
+                    set_gem(cc, gain=ga10*0.99-0.001)
+                    ok = True
+
+
+    # time.sleep(0.2)
+    return
+
+    # 48 - 20 = 28;  28/4 ... 6x
     maxg,gg,ming = 2,1,0
+    gg,mg,ex = 0,0,0
+    defg,defm = 0,0
     maxe,ex,mine = 500,100,100
+    maxg,ex,minm = 500,100,100
+    maxm,ex,minm = 500,100,100
 
     if "gain" in capa:
         gg = cc.get_gain()
         maxg = cc.getmax_gain()
-        ming = cc.getmix_gain()
+        ming = cc.getmin_gain()
+
+    if "gamma" in capa:
+        mg = cc.get_gamma()
+        maxm = cc.getmax_gamma()
+        minm = cc.getmin_gamma()
+        defm = cc.getdef_gamma()
+        maxm = limitgamma # terrible result if too much
 
 
     if "exposure_absolute" in capa:
@@ -425,15 +581,71 @@ def tune_histo(cc, h_avg):
         maxe = cc.getmax_exposure_absolute()
 
 
+    if dist<-1:
+        if "exposure_auto" in capa:
+            exposure_auto = cc.get_exposure_auto()
+            #if exposure_auto!=1:
+            cc.set_exposure_auto(3)
+        else:
+            print("X... cannot tune the exposure (no auto)")
+            return
+        return
+
+    elif dist>4:
+        FAC = 0.5
+    elif dist>3:
+        FAC = 0.7
+    elif dist>2:
+        FAC = 0.9
+    elif dist>1:
+        FAC = 0.98
+    #print(FAC)
+    #print(FAC)
+    # no caf     CAF = 1/FAC
+
+    # CAF = 1/FAC_BASE # I override the speed from bottom to up
+
+    print(f"i...  {med} +- {dg}  <{h_avg}>  {dist:4.1f}sigma  {FAC:5.2f}x")
+
+
+    if "exposure_absolute" in capa:
+        exposure_absolute = cc.get_exposure_absolute()
+        if exposure_absolute!=1:
+            cc.set_exposure_auto(1)
+    else:
+        print("X... cannot tune the exposure (no auto)")
+        return
+
+
+    gg2,ex2,mg2 = gg,ex,mg
+
     #print(f"                   {ming} - {gg} - {maxg} ; {mine}-{ex}-{maxe} ")
-    if h_avg<glow:
-        #print("              +2    ", gg,ex)
-        ex = int(1.1*ex)
-        #print("              +2!    ", gg,ex)
-        gg = int(1.1*gg)
+    if h_avg<glow: #------------------ increase
+        ex2 = int(FAC*ex)
+        gg2 = int(FAC*gg)
+        mg2=mg # tune separate
+        #mg2 = int(1.1*mg)
+        #print(f"increasing {ex} to {ex2}")
+        if gg2==gg:
+            gg2+=1
+        #if mg2==mg:
+        #    mg2+=1
+        if ex2==ex:
+            ex2+=1
+        #print(f"increasing {ex} to {ex2}")
     elif h_avg>ghigh:
-        ex = int(0.9*ex)
-        gg = int(0.9*ex)
+        ex2 = int(FAC*ex)
+        gg2 = int(FAC*gg)
+        mg2 = defm # int(0.9*mg)
+        if gg2==gg:
+            gg2-=1
+        #if mg2==mg:
+        #    mg2-=1
+        if ex2==ex:
+            ex2-=1
+
+    gg,ex,mg = gg2,ex2,mg2
+
 
     if gg>maxg: gg=maxg
     if gg<ming: gg=ming
@@ -441,10 +653,32 @@ def tune_histo(cc, h_avg):
     if ex>maxe: ex=maxe
     if ex<mine: ex=mine
 
-    #print("                  ", gg,ex)
+    if (ex==maxe) and (gg==maxg):
+        print("!... on max")
+        mg2=mg
+        if h_avg<glow: #------------------ increase
+            print("inc gamma now")
+            mg2 = int(1.1*mg)
+            #print(f"increasing {mg} to {mg2}")
+            if mg2==mg:
+                mg2+=1
+            if mg2>maxm: mg2=maxm
+            if mg2<minm: mg2=minm
+    else:
+        mg2=defm
+
+
+
+
+
+    #print(f"                   gain {gg}  expo {ex} gamma {mg} ")
+    #print(f"                   gain {gg}  expo {ex} gamma {mg} ")
+    print(f"        gain {gg} ({ming}/{maxg})  expo {ex} ({mine}/{maxe}) gamma {mg} ({minm}/{maxm}) ")
+    time.sleep(0.6)
 
     if "exposure_absolute" in capa: cc.set_exposure_absolute(ex)
     if "gain" in capa: cc.set_gain(gg)
+    if "gamma" in capa: cc.set_gamma(mg2)
 
     # if gg>maxg:
     #     if h_avg<glow:
