@@ -1,4 +1,4 @@
-#!/usr/bin/env python3
+
 # to override print <= can be a big problem with exceptions
 #from __future__ import print_function # must be 1st
 #import builtins
@@ -11,7 +11,9 @@ from fire import Fire
 from flashcam import config
 config.CONFIG['filename'] = "~/.config/flashcam/cfg.json"
 config.load_config()
-# print(config.CONFIG)
+print("---------------------------------------- LOADED IN WEBPY")
+print(config.CONFIG)
+print(config.CONFIG["histogram"], "==histo in web")
 # print("------------JUST LOADED---------------")
 
 
@@ -471,7 +473,7 @@ def verify_password(username, password):
 #    if not user or not user.verify_password(password):
 #        return False
 #    g.user = user
-    config.load_config()
+    # config.load_config() # IMPLIES THAT ALL changes in 'bin flask500' are lost
     # config.show_config()
     u = config.CONFIG["user"]
     p = config.CONFIG["password"]
@@ -485,11 +487,11 @@ def verify_password(username, password):
     #     print("NO FILE  ","~/.pycamfw_pass")
 
     if (username==u) and (password==p):
-        logthis( "   TRUE  checking userpass (verify)"+username+"/"+password+"/")
+        logthis( "   TRUE  checking userpass (client)"+username+"/"+password+"/")
         logthis( "   TRUE  checking userpass (real  )"+u+"/"+p+"/")
         return True
     else:
-        logthis( "   FALSE checking userpass (verify)"+username+"/"+password+"/")
+        logthis( "   FALSE checking userpass (client)"+username+"/"+password+"/")
         logthis( "   FALSE checking userpass (real  )"+u+"/"+p+"/")
         return False
 
@@ -651,9 +653,12 @@ def gen(camera, remote_ip, blend=False, bigtext=True):
         #===== MAYBE THIS IS WASTING - it should throw get_frame
         #  but with -k sync   it restarts
 
+        timetag = stop.strftime('%Y-%m-%d %H:%M:%S.%f')[:-4]
+        TTAG = f"{framecnt:07d}#{timetag}#"
+        # print(TTAG)
         # yield ( frame)  #--------- JPEG vs MJPEG
         yield (b'--frame\r\n' # ---- JPEG
-               b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')
+               b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n' + b'#FRAME_ACQUISITION_TIME#' + TTAG.encode("utf8") )
 
 
 
